//--------------------------------------------------------------  Alínea A:

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#define MAXK 100

/*	Estrutura de dados:
	- int K; // variável com os dados do jogo, apenas um só número inteiro
	- int sequencia[MAXK]; // vetor para guardar a sequência. Os valores não utilizados são 0.
	Alternativa 1:
	- int sequencia1[MAXK]; // os valores não utilizados não são inicializados
	- int tamanho1; // tamanho da sequência, essencial devido à opção sequencia1
	Alternativa 2:
	- não utilizar estrutura de dados para o vetor, atendendo a que se pretende apenas mostrar
	  o vetor inicial

	Abstrações funcionais:
	void ConstruirSeq(int K, int sequencia[MAXK]);
	void ConstruirSeq1(int K, int sequencia1[MAXK], int *tamanho1);
	- função para construir uma sequência inicial para um dado K
	void MostrarSeq(int sequencia[MAXK]);
	void MostrarSeq1(int sequencia1[MAXK], int tamanho1);
	- função para mostrar uma sequência
	int LerK();
	- função para ler o valor de K de acordo com os limites especificados

	Comentários:
	- a alternativa 2, ao não utilizar estrutura de dados para a sequência, 
	  e utilizar tudo na função main, inviabiliza a reutilização direta de qualquer
	  função. Assim, este código justifica-se caso não seja dada continuação ao
	  projeto, ficando-se por esta alínea. Se é pretendida a continuação, 
	  mesmo sem sequer prever o que será necessário nas alíneas seguintes,
	  a estrutura de dados para guardar a sequência, e a função para mostrar
	  a sequência é de quase certa utilização
	- a solução base e alternativa 1 têm de distinto a opção no vetor, 
	  guardar uma variável com o tamanho (alternativa 1) ou zerar os elementos
	  do vetor não utilizados (solução base). Esta opção de zerar os elementos
	  não utilizados é possível apenas porque a sequência não contém zeros.
	  Ambas as soluções são boas, apenas a solução base fica com menos parametros.
	- A funcionalidade de LerK() poderia ter sido colocada na função main.
	  Ao retirar do main, permitiu a sua reutilização nas restantes alíneas.
	  Por outro lado, deixa a função main sem entrada/saida, ficando a entrada/saída
	  limitada a um conjunto de funções, sendo nessas funções que se tem de trabalhar
	  caso seja necessário a sua alteração. 
	- A funcionalidade de ConstruirSeq() poderia ser também colocada na função main.
	  A inicialização é simples, mas ao colocar numa função permite a 
	  reutilização posterior, e no caso das regras mudarem a inicialização, 
	  bastaria alterar esta função.
*/

void ConstruirSeq(int K, int sequencia[MAXK]) {
	int i;
	sequencia[0] = sequencia[1] = K / 2;
	for (i = 2; i < MAXK; i++)
		sequencia[i] = 0;
}

void MostrarSeq(int sequencia[MAXK]) {
	int i;
	printf("\nSequencia: ");
	for (i = 0; i < MAXK && sequencia[i]>0; i++)
		printf("%d ", sequencia[i]);
}

int LerK() {
	int K;
	printf("Indique K: ");
	scanf("%d", &K);
	if (K < 2 || K>100) {
		printf("\nK tem de ser entre 2 e 100.");
		K = 0;
	}
	return K;
}

void main()
{
	int K;
	int sequencia[MAXK];
	K = LerK();
	if (K > 0) {
		ConstruirSeq(K, sequencia);
		MostrarSeq(sequencia);
	}
}

void ConstruirSeq1(int K, int sequencia1[MAXK], int *tamanho1) {
	sequencia1[0] = sequencia1[1] = K / 2;
	*tamanho1 = 2;
}

void MostrarSeq1(int sequencia1[MAXK], int tamanho1) {
	int i;
	printf("\nSequencia: ");
	for (i = 0; i < tamanho1; i++)
		printf("%d ", sequencia1[i]);
}

void main1()
{
	int K;
	int sequencia1[MAXK], tamanho1;
	K = LerK();
	if (K > 0) {
		ConstruirSeq1(K, sequencia1, &tamanho1);
		MostrarSeq1(sequencia1, tamanho1);
	}
}

int main2()
{
	int K;
	printf("Indique K: ");
	scanf("%d", &K);

	if (K < 2 || K>100)
		printf("K tem de ser entre 2 e 100.");
	else
		printf("Sequencia: %d %d", K / 2, K / 2);
	return 0;
}
/*	Versão do segundo printf com variável auxiliar desnecessária:
	{
		int valor = K / 2;
		printf("Sequencia: %d %d", valor, valor);
	}
*/



//--------------------------------------------------------------  Alínea B:

/*	Estrutura de dados:
	char* resultados[]; // estados possíveis da sequência

	Abstrações funcionais:
	int Soma(int sequencia[MAXK]); int Produto(int sequencia[MAXK]); 
	- soma / multiplica os elemntos da sequência
	int Tamanho(int sequencia[MAXK]);
	- dada a opção da estrutura de dados na solução base da alínea A, é necessário 
	  esta função para facilitar o uso do tamanho da sequência, sempre que seja útil
	int DiferencasABS(int sequencia[MAXK])
	- calcula a difença absoluta entre todos os pares da sequência
	void LerSeq(int sequencia[MAXK]);
	void LerSeq1(int sequencia[MAXK]);
	- ler uma sequência para teste. Embora seja útil apenas nesta alínea, é conveniente
	  estar numa função para manter a função main simples
	int VerificarSeq(int K, int sequencia[MAXK]);
	- retorna 0 caso a sequência seja inválida, 1 caso seja válida, e 2 se for vitória, 
	  implementando as regras do jogo, com recurso às funções elementares implementadas
	void VerificarSeq1(int K, int sequencia[MAXK]);
	- versão procedimento, que imprime logo

	Comentários:
	- A utilização de um código para o resultado, permite a sua reutilização mais tarde, 
	  enquanto que o procedimento VerificarSeq1() não utiliza um código para o resultado, 
	  e imprime logo o que é preciso para esta alínea. Assim, inviabiliza a utilização direta 
	  desta função nas alíneas seguintes. 
	- Houve uma versão intermédia, imprime e retorna código, para permitir a sua reutilização,
	  mas assim mistura lógica do programa com output, inviabiliza a sua utilização caso
	  a lógica se mantenha, e o output se altere.
	- As funções soma/produto/diferençasABS podem estar na função VerificarSeq(), existindo
	  reutilização apenas da função Soma(). Como a sua funcionalidade é clara, fazendo
	  sentido até fora deste programa específico, optou-se por fazer as funções. É também
	  uma boa opção colocar tudo em VerificarSeq(), atendendo a que a dimensão da função
	  é reduzida.
	- a diferença entre LerSeq() e LerSeq1() é a utilização do scanf o gets (ou fgets).
	  Não tendo sido dado detalhes sobre a formatação dos dados de entrada, apenas inteiros,
	  é preferivel utilizar o scanf, que se pode solicitar inteiros, ignorando o resto.
	  Nas atividades formativas utiliza-se sempre o scanf nestes casos.
	  A utilização do gets() e strtok() assume que os dados vêm numa só linha, que é o que acontece
	  nos casos de teste, mas não é garantido no enunciado e é menos genérico, e também mais complexo.
	- main1() utiliza a função VerifcarSeq() mas não a estrutura de dados com o texto dos
	  resultados possíveis. Esta opção conduz a uma cadeia if-else (poderia ser um switch),
	  que seria evitado com o vetor com o texto de todos os resultados possíveis.
	- o código desta alínea poderia estar também numa só função main, impedindo qualquer
	  reutilização direta do código, e elevando demasiado o tamanho da função. Já na alínea
	  A esta opção não é a ideal devido a não servir para reutilização, 
	  aqui mais desaconselhada é atendendo à dimensão da função não facilitar a leitura do código.

*/

int Soma(int sequencia[MAXK]) {
	int i, soma = 0;
	for (i = 0; i < MAXK && sequencia[i]>0; i++)
		soma += sequencia[i];
	return soma;
}

int Produto(int sequencia[MAXK]) {
	int i, produto = 1;
	for (i = 0; i < MAXK && sequencia[i]>0; i++)
		produto *= sequencia[i];
	return produto;
}

int Tamanho(int sequencia[MAXK]) {
	int i;
	for (i = 0; i < MAXK && sequencia[i]>0; i++);
	return i;
}

int DiferencasABS(int sequencia[MAXK]) {
	int i, j, diferencas = 0, tamanho;
	tamanho = Tamanho(sequencia);
	for (i = 0; i < tamanho - 1; i++)
		for (j = i + 1; j < tamanho; j++)
			diferencas += abs(sequencia[i] - sequencia[j]);
	return diferencas;
}

void LerSeq(int sequencia[MAXK]) {
	int i;
	ConstruirSeq(0, sequencia);
	printf("\nIndique uma sequencia de numeros inteiros positivos, terminando com 0: ");
	i = 0;
	do {
		if (scanf("%d", &(sequencia[i])) == 0)
			sequencia[i] = 0;
		i++;
	} while (sequencia[i - 1] > 0 && i < MAXK);
}

int VerificarSeq(int K, int sequencia[MAXK]) {
	if (Soma(sequencia) > K)
		return 0; // sequência inválida
	if (Produto(sequencia) < K)
		return 0;
	if (DiferencasABS(sequencia) == K)
		return 2; // sequência de vitória
	return 1; // sequência válida
}
void main()
{
	int K;
	int sequencia[MAXK];
	char* resultados[] = { "invalida","valida","vitoria" };
	K = LerK();
	if (K > 0) {
		LerSeq(sequencia);
		printf("\nSequencia %s", resultados[VerificarSeq(K, sequencia)]);
	}
}

void LerSeq1(int sequencia[MAXK]) {
	char str[255], *pt; // assumir a entrada numa só linha
	int i;
	ConstruirSeq(0, sequencia);
	printf("\nIndique uma sequencia de numeros inteiros positivos, terminando com 0: ");
	i = 0;
	gets(str);
	pt = strtok(str, " \t\n\r");
	while (pt != NULL && i < MAXK) {
		sequencia[i] = atoi(pt);
		if (sequencia[i] > 0)
			i++;
		else
			i = MAXK; // terminar a leitura
		pt = strtok(NULL, " \t\n\r");
	}
}

void main1()
{
	int K, resultado;
	int sequencia[MAXK];
	K = LerK();
	if (K > 0) {
		LerSeq1(sequencia);
		resultado = VerificarSeq(K, sequencia);
		if (resultado == 2)
			printf("\nSequencia vitoria");
		else if (resultado == 1)
			printf("\nSequencia valida");
		else
			printf("\nSequencia invalida");
	}
}

void VerificarSeq1(int K, int sequencia[MAXK]) {
	int i, soma = 0, produto=1, diferencas=0, tamanho;
	for (i = 0; i < MAXK && sequencia[i]>0; i++) {
		soma += sequencia[i];
		produto *= sequencia[i];
		if (soma > K) {
			printf("\nSequencia invalida");
			return;
		}
	}
	if (produto < K) {
		printf("\nSequencia invalida");
		return;
	}
	tamanho = i;
	for (i = 0; i < tamanho - 1; i++)
		for (j = i + 1; j < tamanho; j++)
			diferencas += abs(sequencia[i] - sequencia[j]);
	if (diferencas == K) 
		printf("\nSequencia vitoria");
	else
		printf("\nSequencia valida");
}

void main2()
{
	int K, resultado;
	int sequencia[MAXK];
	K = LerK();
	if (K > 0) {
		LerSeq(sequencia);
		VerificarSeq1(K, sequencia);
	}
}


//--------------------------------------------------------------  Alínea C:

/*	Estrutura de dados:
	int jogada[2]; // jogada são dois inteiros
	int jogador; // código com o jogador a jogar 0/1
	int njogada; // número da jogada atual
	int manterJogador; // indicação se o jogador deve manter a jogada ou não 

	Abstrações funcionais:
	int LerJogada(int jogada[2]);
	- le uma jogada
	void Remover(int indice, int sequencia[MAXK]);
	void Inserir(int indice, int valor, int sequencia[MAXK])
	- remove / insere um elemento na sequência
	int ExecutarJogada(int K, int sequencia[MAXK], int jogada[2], int *manterJogador);
	- executa uma jogada na sequência, verificando se mantém o jogador ou não na vez de jogar
	void JogoSequencia();
	- executa o jogo

	Comentários:
	- houve outros códigos de jogador, como 'A'/'B'. Os códigos, de um modo geral, é sempre
	  mais simples que sejam a começar em 0 e terminar no número de elementos. Pode-se 
	  ter depois um vetor com informação de cada código. Neste caso, como 'A' e 'B' são 
	  códigos ASCII seguidos, podemos utilizar a expressão 'A'+jogador para converter o 
	  código no elemento que representa. A vantagem do código ser conhecido, neste caso
	  0 ou 1, é que podemos utilizar expressões para os mudar, como trocar de código, 
	  será jogador=1-jogador; Se o código não for conhecido, a troca terá de ser 
	  if(jogador=='A') jogador='B'; else jogador='A'; mais longo
	- LerJogada() mantém a interface num conjunto de funções separadas da lógica do jogo
	- Remover() / Inserir() são funções de utilização genérica, pelo que se optou por definir,
	  pelo mesmo motivo que se implementou Soma/Produto/DiferencasABS, embora pudessem estar 
	  em ExecutarJogada
	- ExecutarJogada() é a função principal desta alínea, naturalmente para ser reutilizada 
	  na alínea seguinte, é essencial que esteja numa função separada
	- JogoSequencia() optou-se por fazer fora da função main, mas atendendo a que a função main
	  ficou com apenas esta função, é uma opção questionável

*/

int LerJogada(int jogada[2]) {
	printf("\nJogada (indice valor): ");
	if (scanf("%d %d", &(jogada[0]), &(jogada[1])) == 2)
		return 1;
	return 0;
}

void Remover(int indice, int sequencia[MAXK]) {
	int i;
	// remover posição i, deslocando os elementos após i para a esquerda
	for (i = indice; i < MAXK - 1 && sequencia[i]>0; i++)
		sequencia[i] = sequencia[i + 1];
}

void Inserir(int indice, int valor, int sequencia[MAXK]) {
	int i;
	// deslocar os elementos após i para a direita, e atribuir novo valor
	for (i = MAXK - 1; i > indice; i--)
		sequencia[i] = sequencia[i - 1];
	sequencia[indice] = valor;
}

int ExecutarJogada(int K, int sequencia[MAXK], int jogada[2], int *manterJogador) {
	int i;
	int indice = jogada[0], valor = jogada[1];
	// avançar até ao índice (ou até ao fim da sequência)
	for (i = 0; i < indice && sequencia[i]>0 && i < MAXK; i++);
	if (valor > 0) {
		sequencia[i] = valor; // substituir o valor
		*manterJogador = 0; // passa a vez sempre
	}
	else {
		if (valor == 0) // remover índice i 
			Remover(i, sequencia);
		else // inserir -valor na posição
			Inserir(i, -valor, sequencia);
		*manterJogador = 1 - *manterJogador; // se já estava a ser mantido, deixar de estar, c.c. manter 
	}
	return VerificarSeq(K, sequencia);
}

void JogoSequencia() 
{	// colocado em função, para reutilização na alínea D
	int K, resultado, jogador = 1, njogada = 0, manterJogador = 0;
	int sequencia[MAXK], jogada[2];
	K = LerK();
	if (K > 0) {
		ConstruirSeq(K, sequencia);
		do {
			if (manterJogador == 0)
				jogador = 1 - jogador;
			MostrarSeq(sequencia);
			printf(" [Joga %c] ", 'A' + jogador);
			if (!LerJogada(jogada))
				resultado = 0;
			else
				resultado = ExecutarJogada(K, sequencia, jogada, &manterJogador);
			njogada++;
		} while (resultado == 1 && njogada < K);
		MostrarSeq(sequencia); // última sequência
		if (resultado == 0)
			printf("\nJogador %c perdeu.", 'A' + jogador);
		else if (njogada == K)
			printf("\nEmpate.");
		else
			printf("\nJogador %c ganhou.", 'A' + jogador);
	}
}

void main()
{
    JogoSequencia(); 
}


//--------------------------------------------------------------  Alínea D:

#define MAXNJOG 10000

/*	Estrutura de dados:
	int jogadas[MAXNJOG][2]; // vetor com jogadas 
	int indicesTestados[MAXK]; // vetor com índices testados, para não repetir

	Abstrações funcionais:
	int CriarJogadas(int K, int indice, int sequencia[MAXK], int jogadas[MAXNJOG][2]);
	- cria as jogadas possíveis com base num índice, colocando-as numa estrutura
	int MelhorJogada(int K, int njogadas, int sequencia[MAXK], int jogadas[MAXNJOG][2]);
	- verifica quais são as melhores jogadas
	void JogadorArtificial(int K, int sequencia[MAXK], int jogada[2]);
	- escolhe o índice, criando as jogadas e escolhendo a melhor jogada de acordo com as indicações
	int LerJogadaGeral(int K, int sequencia[MAXK], int jogada[2]);
	- verifica se a jogada é -2 -2, e nesse caso executa o jogador artificial para gerar uma jogada
	void JogoSequencia();
	- função atualizada da alínea anterior, para utilizar LerJogadaGeral()

	Alternativa a processar as jogadas primeiro:
	int RegistarJogada(int melhorJog[2], int jogada[2], int sequencia[MAXK]);
	- verifica se a jogada[2] é melhor que a atual, se sim substitui melhorJog[2]
	- se vitória, retorna 1 de modo a parar o processamento
	void MelhorJogada1(int K, int indice, int sequencia[MAXK], int melhorJog[2]);
	- funcionamento idêntico, mas não recebe as jogadas, faz as jogadas na própria função
	- esta função acaba por somar CriarJogadas e MelhorJogadas, mas pode ser separada com RegistarJogada()
	void JogadorArtificial1(int K, int sequencia[MAXK], int jogada[2]);
	- chama MelhorJogada1() não precisando de criar jogadas antes

	Comentários:
	- a criação da estrutura com as jogadas é uma decisão importante, para permitir 
	  a reutilização da execução das jogadas, e para permitir separar a geração das jogadas da 
	  análise da melhor jogada
	- a função com a melhor jogada é assim apenas a execução das jogadas criadas, reutilizando
	  o código já existente
	- ao criar as jogadas, apenas interessam os limites a colocar no valor, sendo o índice fixo
	- no jogador artificial apenas itera o índice, gerando as jogadas e optando pela melhor jogada
	  com base nas restantes funções implementadas
    - a altarnativa 1 não utiliza a estrutura com as jogadas temporária, sendo mais eficiente em termos
	  de memória, e é uma alternativa equivalente à solução base
	- há soluções que não desagregaram em funções uma destas versões, tanto numa alternativa como na 
	  outra, conduzindo a funções muito grandes. Evidentemente que esta solução é completamente
	  desaconselhada, não permitindo a fácil leitura de código, e inviabilizando qualquer tipo de 
	  possibilidade de reutilização, dificultando (ou inviabilizando) a manutenção do código.
	  Se o código não puder ser mantido, o resultado é que tem de ser refeito assim que apareça
	  algum problema, perdendo-se o trabalho de construir 

*/


int CriarJogadas(int K, int indice, int sequencia[MAXK], int jogadas[MAXNJOG][2]) {
	int njogadas = 0, i, soma;
	// soma A de todos os valores
	soma = Soma(sequencia);
	// considerar as possibilidades de remover um número ou reduzir
	for (i = 0; i < sequencia[indice]; i++) {
		jogadas[njogadas][0] = indice;
		jogadas[njogadas][1] = i; // novo valor
		njogadas++;
		if (njogadas == MAXNJOG)
			return MAXNJOG;
	}
	if (soma < K) { // considerar também as possibilidades de adicionar um número ou aumentar o atual
		for (i = sequencia[indice]+1; i <= sequencia[indice]+(K-soma); i++) {
			jogadas[njogadas][0] = indice;
			jogadas[njogadas][1] = i; // novo valor
			njogadas++;
			if (njogadas == MAXNJOG)
				return MAXNJOG;
		}
		// colocar novo número
		for (i = 1; i <= K - soma; i++) {
			jogadas[njogadas][0] = indice;
			jogadas[njogadas][1] = -i; // novo valor, negativo para inserir novo número
			njogadas++;
			if (njogadas == MAXNJOG)
				return MAXNJOG;
		}
	}
	return njogadas;
}

int MelhorJogada(int K, int njogadas, int sequencia[MAXK], int jogadas[MAXNJOG][2]) {
	int i, j, resultado, jogada=-1, tamanho=-1, manterJogador=0;
	int seqAux[MAXK];
	// percorrer todas as jogadas
	for (i = 0; i < njogadas; i++) {
		for (j = 0; j < MAXK; j++)
			seqAux[j] = sequencia[j];
		resultado = ExecutarJogada(K, seqAux, jogadas[i], &manterJogador);
		if (resultado == 2) // ganha
			return i;
		if (resultado == 1) { // lance válido
			if (tamanho == -1 || tamanho > Tamanho(seqAux)) {
				tamanho = Tamanho(seqAux);
				jogada = i;
			}
			// não é preciso ter em consideração a soma de todos os números mais baixo
			// atendendo a que as jogadas são criadas por essa ordem
		}
	}
	return jogada;
}

void JogadorArtificial(int K, int sequencia[MAXK], int jogada[2]) {
	int i, indice, valor;
	int jogadas[MAXNJOG][2];
	int njogadas;
	int indicesTestados[MAXK];
	for (i = 0; i < MAXK; i++)
		indicesTestados[i] = 0;
	
	// índice I com o menor valor na sequência

	do {
		for (i = 0, indice = -1; i < MAXK && sequencia[i]>0; i++)
			if ((indice<0 || valor < sequencia[i]) && indicesTestados[i]==0) {
				valor = sequencia[i];
				indice = i;
			}
		if (indice < 0)
			break;
		//printf("\nTeste indice %d.", indice);
		indicesTestados[indice] = 1; // não testar novamente
		njogadas = CriarJogadas(K, indice, sequencia, jogadas);
		i = MelhorJogada(K, njogadas, sequencia, jogadas);
	} while (i < 0); // todas as jogadas inválidas

	if (i < 0) {
		// todas as jogadas inválidas, não fazer nada
		jogada[0] = 0;
		jogada[1] = sequencia[0]; // não faz nada
	}
	else {
		jogada[0] = jogadas[i][0];
		jogada[1] = jogadas[i][1];
	}
}

int LerJogadaGeral(int K, int sequencia[MAXK], int jogada[2]) {
	int resultado = LerJogada(jogada);
	if (resultado && jogada[0] == -2 && jogada[1] == -2) {
		JogadorArtificial(K, sequencia, jogada);
		printf("\nJogada artificial: %d %d", jogada[0], jogada[1]);
	}
	return resultado;
}


void JogoSequencia()
{	
	int K, resultado, jogador = 1, njogada = 0, manterJogador = 0;
	int sequencia[MAXK], jogada[2];
	K = LerK();
	if (K > 0) {
		ConstruirSeq(K, sequencia);
		do {
			if (manterJogador == 0)
				jogador = 1 - jogador;
			MostrarSeq(sequencia);
			printf(" [Joga %c] ", 'A' + jogador);
			if (!LerJogadaGeral(K, sequencia, jogada)) ///// única alteração para a alínea C
				resultado = 0;
			else
				resultado = ExecutarJogada(K, sequencia, jogada, &manterJogador);
			njogada++;
		} while (resultado == 1 && njogada < K);
		MostrarSeq(sequencia); // última sequência
		if (resultado == 0)
			printf("\nJogador %c perdeu.", 'A' + jogador);
		else if (njogada == K)
			printf("\nEmpate.");
		else
			printf("\nJogador %c ganhou.", 'A' + jogador);
	}
}

void main()
{
	JogoSequencia();
}

int RegistarJogada(int K, int melhorJog[2], int jogada[2], int sequencia[MAXK]) {
	int j, seqAux[MAXK], resultado, manterJogador = 0;
	for (j = 0; j < MAXK; j++)
		seqAux[j] = sequencia[j];
	resultado = ExecutarJogada(K, seqAux, jogada, &manterJogador);
	if (resultado == 2) {
		melhorJog[0] = jogada[0];
		melhorJog[1] = jogada[1];
		return 1; // terrminar
	}
	else if (resultado == 1 && melhorJog[0] == -2) { // primeira jogada válida
		melhorJog[0] = jogada[0];
		melhorJog[1] = jogada[1];
	}
	return 0;
}

void MelhorJogada1(int K, int indice, int sequencia[MAXK], int melhorJog[2]) {
	int soma;
	int i, j;
	int jogada[2];

	soma = Soma(sequencia);
	melhorJog[0] = melhorJog[1] = -2;

	// considerar as possibilidades de remover um número ou reduzir
	for (i = 0; i < sequencia[indice]; i++) {
		jogada[0] = indice;
		jogada[1] = i; // novo valor
		if (RegistarJogada(K, melhorJog, jogada, sequencia))
			return;
	}
	if (soma < K) { // considerar também as possibilidades de adicionar um número ou aumentar o atual
		for (i = sequencia[indice] + 1; i <= sequencia[indice] + (K - soma); i++) {
			jogada[0] = indice;
			jogada[1] = i; // novo valor
			if (RegistarJogada(K, melhorJog, jogada, sequencia))
				return;
		}
		// colocar novo número
		for (i = 1; i <= K - soma; i++) {
			jogada[0] = indice;
			jogada[1] = -i; // novo valor, negativo para inserir novo número
			if (RegistarJogada(K, melhorJog, jogada, sequencia))
				return;
		}
	}
}

void JogadorArtificial1(int K, int sequencia[MAXK], int jogada[2]) {
	int i, indice, valor;
	int njogadas;
	int indicesTestados[MAXK];
	for (i = 0; i < MAXK; i++)
		indicesTestados[i] = 0;

	// índice I com o menor valor na sequência

	do {
		for (i = 0, indice = -1; i < MAXK && sequencia[i]>0; i++)
			if ((indice < 0 || valor < sequencia[i]) && indicesTestados[i] == 0) {
				valor = sequencia[i];
				indice = i;
			}
		if (indice < 0)
			break;
		//printf("\nTeste indice %d.", indice);
		indicesTestados[indice] = 1; // não testar novamente
		MelhorJogada1(K, indice, sequencia, jogada);
	} while (jogada[0] == -2); // todas as jogadas inválidas

	if (jogada[0] == -2) {
		// todas as jogadas inválidas, não fazer nada
		jogada[0] = 0;
		jogada[1] = sequencia[0]; // não faz nada
	}
}
